package speiger.src.coreengine;

import java.util.List;

import org.lwjgl.glfw.GLFW;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL15;

import speiger.src.coreengine.assets.AssetLocation;
import speiger.src.coreengine.assets.AssetManager;
import speiger.src.coreengine.assets.base.IAssetPackage;
import speiger.src.coreengine.assets.base.IAssetProvider;
import speiger.src.coreengine.rendering.input.devices.FileDrop;
import speiger.src.coreengine.rendering.input.devices.Joystick;
import speiger.src.coreengine.rendering.input.devices.Keyboard;
import speiger.src.coreengine.rendering.input.devices.Mouse;
import speiger.src.coreengine.rendering.input.window.Window;
import speiger.src.coreengine.rendering.input.window.WindowManager;
import speiger.src.coreengine.rendering.shader.Shader;
import speiger.src.coreengine.rendering.shader.SimpleShader;
import speiger.src.coreengine.rendering.shader.uniform.base.TextureUniform;
import speiger.src.coreengine.rendering.utils.GLStateTracker;
import speiger.src.coreengine.utils.eventbus.EventBus;
import speiger.src.coreengine.utils.helpers.IOUtils;

public class NewInputTest {
	
	EventBus bus = new EventBus();
	WindowManager manager = new WindowManager();
	AssetManager assets = new AssetManager(List.of(IAssetPackage.of(IOUtils.getBaseLocation())));
	private Shader<TestShader> shaderTest = Shader.create(TestShader::new);
	
	
	public static void main(String[] args) {
		new NewInputTest().run();
	}
	
	public void run() {
		GLFW.glfwInit();
		manager.initialize();
		Mouse.INSTANCE.init(bus);
		Keyboard.INSTANCE.init(bus);
		Joystick.INSTANCE.init(manager, bus);
		FileDrop.INSTANCE.init(bus);
		manager.addDevices(Mouse.INSTANCE, Keyboard.INSTANCE, Joystick.INSTANCE, FileDrop.INSTANCE);
		Window window = manager.builder().title("Testing Engine").build();
		shaderTest.register();
		assets.addListener(GLStateTracker.instance().shaders);
		assets.reload(Runnable::run, Runnable::run);
		window.visible(true);
		while(!window.shouldClose()) {
			GLFW.glfwPollEvents();
			window.beginFrame();
			window.handleInput();
			window.finishFrame();
			try { Thread.sleep(100); }
			catch(InterruptedException e) { e.printStackTrace(); }
		}
		window.destroy();
		manager.destroy();
	}
	
	public static class TestShader extends SimpleShader {
		public TextureUniform texture = uniforms.addTexture("texture", 0);
		public TestShader(IAssetProvider provider) {
			super(provider, "testing_shader", AssetLocation.of("shader/testing/vertex.vs"), AssetLocation.of("shader/testing/fragment.fs"), "in_position", "in_tex", "in_color");
		}
	}
}
